/* *********************************************************************
 * Class EpicurDAOImpl
 * 
 * Copyright (c) 2010-2013, German National Library/Deutsche Nationalbibliothek
 * Adickesallee 1, D-60322 Frankfurt am Main, Federal Republic of Germany
 * 
 * This program is free software. Licensed under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 * 
 * Kadir Karaca Kocer -- German National Library
 * 
 **********************************************************************/

/* ********************************************************************
 * CHANGELOG:
 * 
 * 2013-02-26 More comments and Checkstyle improvements, Karaca
 * 2012-03-15 Commented and ported to Apache Maven, Karaca
 * 2012-10-01 getNamespaceByName() and getNamespaceIdsForInstitution() added, Karaca
 * 2012-08-30 New function getUrlsFaultyByInstId() - M. Klein
 * 2012-08-29 New functions getNumberOf...ByInstId() - M. Klein
 * 2012-08-28 Functions getNumberOf...ByPersId() deleted - M. Klein
 * 2012-07-16 Refactoring - M. Klein
 * 2012-07-10 New functions getNumberOf...ByPersId() - M. Klein
 * 2012-07-09 Adjusting the interface - M. Klein
 * Created on 2011-08-25 by Kadir Karaca Kocer, German National Library 
 * ******************************************************************
 */

package org.nbnResolving.database.impl;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.nbnResolving.database.dao.IEpicurDAO;
import org.nbnResolving.database.model.JoinNSPERMISSION;
import org.nbnResolving.database.model.TableINSTITUTION;
import org.nbnResolving.database.model.TableMIMETYPES;
import org.nbnResolving.database.model.TableNAMESPACE;
import org.nbnResolving.database.model.TableNS2PERMISSION;
import org.nbnResolving.database.model.TablePERMISSION;
import org.nbnResolving.database.model.TablePERSON;
import org.nbnResolving.database.model.TableURL;
import org.nbnResolving.database.model.TableURN;

/**
 * Class that implements the Epicur Data Object Access Interface.
 * This is original Sybase relational database model of
 * the first generation URN:NBN system of German National Library.
 * 
 * Because of this legacy production database, the new generation
 * URN system must be backwards compatible.
 * Please see the original relational data model
 * to understand how this functions and why.
 * 
 * @author Kadir Karaca Kocer, German National Library
 */

public class EpicurDAOImpl implements IEpicurDAO {
    private static final Log LOGGER = LogFactory.getLog(EpicurDAOImpl.class);

    /** Read-Only datasource for performance optimized selects  */
    private DataSource readOnlyDataSource;

    /** Data Source for Read / Write / Update / Delete operations */
    private DataSource writableDataSource;

    // first define the database connections
    private Connection    roConn = null; // read-only connection for high performance selects
    private Connection    rwConn = null; // read|write connection for inserts, updates and deletes

    // getters & setters
    /**
     * @return The Read-Only Data Source
     */
    public DataSource getReadOnlyDataSource() {
        return this.readOnlyDataSource;
    }

    /**
     * @param ds The Data Source to set.
     * @throws SQLException
     */
    public void setReadOnlyDataSource(final DataSource ds) throws SQLException {
        final Connection conn = ds.getConnection();
        if (conn != null) {
            this.roConn = conn;
            this.roConn.setReadOnly(true);
            // this.roConn.set //other options to improve performance?
        } else {
            throw new SQLException("ERROR connecting database. Read Only Database Connection can not be NULL!");
        }
    }

    /**
     * @return The Read & Write Data Source.
     */
    public DataSource getWritableDataSource() {
        return this.writableDataSource;
    }

    /**
     * @param ds The Data Source to set.
     * @throws SQLException
     */
    public void setWritableDataSource(final DataSource ds) throws SQLException {
        final Connection conn = ds.getConnection();
        if (conn != null) {
            this.rwConn = conn;
            this.rwConn.setReadOnly(false);
            // this.rwConn.setAutoCommit(?)
            // this.rwConn.setHoldability(?)
            // this.rwConn.setTransactionIsolation(?)
            // other options to improve performance ?
        } else {
            throw new SQLException("ERROR connecting database. Read|Write Database Connection can not be NULL!");
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TableINSTITUTION getInstitutionById(final int instId) {
        TableINSTITUTION inst = null;
        try {
            inst = DatabaseUtils.getInstitution(this.roConn, instId, true);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getInstitutionById! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return inst;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TableMIMETYPES getMimetypeById(final int mimetypeId) {
        final TableMIMETYPES mime = new TableMIMETYPES();
        try {
            mime.setMimetype_id(mimetypeId);
            mime.setMimetype(DatabaseUtils.getMimeType(this.roConn, mimetypeId));
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getMimetypeById! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return mime;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableMIMETYPES> getMimetypes() {
        List<TableMIMETYPES> mimetypes = null;
        try {
            mimetypes = DatabaseUtils.getMimeTypeList(this.roConn);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getMimetypes! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return mimetypes;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableNS2PERMISSION> getNs2PermissionsByPersId(final int personId) {
        List<TableNS2PERMISSION> n2p = null;
        try {
            n2p = DatabaseUtils.getNs2PermissionsByPersId(this.roConn, personId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNs2PermissionsByPersId! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return n2p;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TableNAMESPACE getNamespaceById(final int nsId) {
        TableNAMESPACE ns = null;
        try {
            ns = DatabaseUtils.getNamespace(this.roConn, nsId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNamespace! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return ns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TableNAMESPACE getNamespaceByName(final String name) {
        TableNAMESPACE ns = null;
        try {
            ns = DatabaseUtils.getNamespace(this.roConn, name);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNamespaceByName! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return ns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Integer> getNamespaceIdsForInstitution(final int instId) {
        List<Integer> list = null;
        try {
            list = DatabaseUtils.getNamespaceIdsForInstitution(this.roConn, instId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNamespaceIdsForInstitution! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return list;
    }
    

    /**
     * {@inheritDoc}
     */
    @Override
    public TablePERMISSION getPermissionById(final int permissionId) {
        TablePERMISSION permission = null;
        try {
            permission = DatabaseUtils.getPermissionById(this.roConn, permissionId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getPermissionById! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return permission;
    }

    // **** PERSON ****************************************
    /**
     * {@inheritDoc}
     */
    @Override
    public TablePERSON getPersonByLogin(final String login) {
        TablePERSON person = null;
        try {
            person = DatabaseUtils.getPersonByLogin(this.roConn, login);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getPersonByLogin! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return person;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TablePERSON getPersonById(final int persId) {
        TablePERSON person = null;
        try {
            person = DatabaseUtils.getPersonById(this.roConn, persId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getPersonById! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return person;
    }

    // **** URL ******************************************
    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURL> getUrlsByUrnId(final long urnId) {
        List<TableURL> urls = null;
        try {
            urls = DatabaseUtils.getURLsByURNid(this.roConn, urnId, false);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrlsByUrnId! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }

        return urls;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURL> getUrlsFaultyByPersId(final int persId) {
        List<TableURL> urls = null;
        try {
            urls = DatabaseUtils.getURLsByPersId(this.roConn, persId, true);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrlsFaultyByPersId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urls;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURL> getUrlsFaultyByInstId(final int instId) {
        List<TableURL> urls = null;
        try {
            urls = DatabaseUtils.getURLsByInstId(this.roConn, instId, true);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrlsFaultyByInstId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urls;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<JoinNSPERMISSION> getNsPermissionsByPersId(final int persId) {
        List<JoinNSPERMISSION> nsPermissions = null;
        try {
            nsPermissions = DatabaseUtils.getNsPermissionsByPersId(this.roConn, persId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNsPermissionsByPersId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return nsPermissions;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURL> getUrlsFaultyByUrnId(final long urnId) {
        List<TableURL> urls = null;
        try {
            urls = DatabaseUtils.getURLsByURNid(this.roConn, urnId, true);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrlsFaultyByUrnId! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }

        return urls;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrlsByUrnId(final long urnId) {
        int count = -1;
        try {
            count = DatabaseUtils.getNumberOfURLsByUrnId(this.roConn, urnId, false);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrlsByUrnId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }

        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrlsByUrl(final String urlSubStr) {
        int count = -1;
        try {
            count = DatabaseUtils.getNumberOfURLsByUrl(this.roConn, urlSubStr);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrlsByUrl()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }

        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrlsFaultyByUrnId(final long urnId) {
        int count = -1;
        try {
            count = DatabaseUtils.getNumberOfURLsByUrnId(this.roConn, urnId, true);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrlsFaultyByUrnId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }

        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURL> getUrlsByUrl(final String urlSubStr) {
        List<TableURL> urls = null;
        try {
            urls = DatabaseUtils.getURLsStartingWith(this.roConn, urlSubStr);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrlsByUrl()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urls;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TableURL getUrlByUrl(final String urlStr) {
        TableURL url = null;
        try {
            url = DatabaseUtils.getURL(this.roConn, urlStr);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrlByUrl()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return url;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void updateUrl(final TableURL url) {
        // TODO check validity
        try {
            AdminUtils.updateURL(url, this.rwConn);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.updateUrl()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void updateUrl(final TableURL url, final String urlStr) {
        // TODO check validity
        try {
            AdminUtils.updateURL(url, urlStr, this.rwConn);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.updateUrl()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void insertUrl(final TableURL url) {
        try {
            AdminUtils.writeURLIntoDB(url, this.rwConn);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.insertUrl()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void deleteUrl(final String urlStr) {
        //Authorisation & Institution id check should happen at a level higher!
        try {
            AdminUtils.deleteURL(urlStr, this.rwConn);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.deleteUrl()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
    }

    // **** URN ******************************************
    /**
     * {@inheritDoc}
     */
    @Override
    public TableURN getUrnById(final long urnId) {
        TableURN urn = null;
        try {
            urn = DatabaseUtils.getURNInfo(this.roConn, urnId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnById()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urn;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURN> getUrnsByNsId(final int nsId) {
        List<TableURN> urns = null;
        try {
            urns = DatabaseUtils.getURNsForNamespace(this.roConn, nsId, DatabaseUtils.URN_ALL);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnsByNsId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURN> getUrnsReservedByNsId(final int nsId) {
        List<TableURN> urns = null;
        try {
            urns = DatabaseUtils.getURNsForNamespace(this.roConn, nsId, DatabaseUtils.URN_RESERVED);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnsReservedByNsId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURN> getUrnsRegisteredByNsId(final int nsId) {
        List<TableURN> urns = null;
        try {
            urns = DatabaseUtils.getURNsForNamespace(this.roConn, nsId, DatabaseUtils.URN_ACTIVE);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnsRegisteredByNsId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURN> getUrnsByUrn(final String urnSubstr) {
        List<TableURN> urns = null;
        try {
            urns = DatabaseUtils.getURNsStartingWith(this.roConn, urnSubstr, DatabaseUtils.URN_ALL);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnsByUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURN> getUrnsReservedByUrn(final String urnSubstr) {
        List<TableURN> urns = null;
        try {
            urns = DatabaseUtils.getURNsStartingWith(this.roConn, urnSubstr, DatabaseUtils.URN_RESERVED);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnsReservedByUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<TableURN> getUrnsRegisteredByUrn(final String urnSubStr) {
        List<TableURN> urns = null;
        try {
            urns = DatabaseUtils.getURNsStartingWith(this.roConn, urnSubStr, DatabaseUtils.URN_ACTIVE);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnsRegisteredByUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urns;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TableURN getUrnByUrn(final String urnStr) {
        TableURN urn = null;
        try {
            urn = DatabaseUtils.getURNInfo(this.roConn, urnStr, false);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getUrnByUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urn;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsByUrn(final String urnSubStr) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNs(this.roConn, urnSubStr, DatabaseUtils.URN_ALL);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrnsByUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsRegisteredByUrn(final String urnSubStr) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNs(this.roConn, urnSubStr, DatabaseUtils.URN_ACTIVE);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrnsRegisteredByUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsReservedByUrn(final String urnSubStr) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNs(this.roConn, urnSubStr, DatabaseUtils.URN_RESERVED);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrnsReservedByUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsReservedByNsId(final int nsId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNs(this.roConn, nsId, DatabaseUtils.URN_RESERVED);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrnsReservedByNsId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsRegisteredByInstId(final int instId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNsByInstId(this.roConn, instId, DatabaseUtils.URN_ACTIVE);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfURNsByInstitutionId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfNamespacesByInstId(final int instId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfNamespacesByInstId(this.roConn, instId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfNamespacesByInstId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsReservedByInstId(final int instId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNsByInstId(this.roConn, instId, DatabaseUtils.URN_RESERVED);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrnsReservedByInstId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsByInstId(final int instId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNsByInstId(this.roConn, instId);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrnsByInstId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrlsByInstId(final int instId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURLsByInstId(this.roConn, instId, false);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrlsByInstId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrlsFaultyByInstId(final int instId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURLsByInstId(this.roConn, instId, true);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrlsFaultyByInstId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfUrnsRegisteredByNsId(final int nsId) {
        int count = -1; // -1: not defined
        try {
            count = DatabaseUtils.getNumberOfURNs(this.roConn, nsId, DatabaseUtils.URN_ACTIVE);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.getNumberOfUrnsRegisteredByNsId()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return count;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void updateUrn(final TableURN urn) {
        try {
            AdminUtils.updateURN(urn, this.rwConn);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.updateUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int insertUrn(final TableURN urn) {
        int urnid = -1; // -1: not defined
        try {
            urnid = (int) AdminUtils.writeURNIntoDB(urn, this.rwConn);
        } catch (SQLException e) {
            LOGGER.error("Exception in EpicurDAOImpl.insertUrn()! Cause:\n" + e.getMessage());
            LOGGER.error("SQL State is:\n" + e.getSQLState());
            e.printStackTrace();
        }
        return urnid;
    }

}
